@@ -13,14 +13,15 @@
 #
 
 import atexit
+import collections
 import copy
 import datetime
 import functools
-import multiprocessing
 import operator
 import signal
 import threading
 import types
+import uuid
 
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
@@ -32,15 +33,15 @@ from neutron_lib import constants as const
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import availability_zone as az_exc
+from neutron_lib.placement import utils as place_utils
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
 from neutron_lib.utils import helpers
-from oslo_concurrency import lockutils
+from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
 from oslo_utils import timeutils
-from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron._i18n import _
 from neutron.common.ovn import acl as ovn_acl
@@ -55,10 +56,13 @@ from neutron.db import provisioning_blocks
 from neutron.extensions import securitygroup as ext_sg
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent as n_agent
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.extensions \
+    import placement as placement_ext
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import maintenance
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_db_sync
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import worker
 from neutron import service
 from neutron.services.logapi.drivers.ovn import driver as log_driver
@@ -71,6 +75,9 @@ import neutron.wsgi
 LOG = log.getLogger(__name__)
 OVN_MIN_GENEVE_MAX_HEADER_SIZE = 38
 
+# TODO(ralonsoh): rehome this to ``neutron_lib.placement.constants``.
+ALLOCATION = 'allocation'
+
 
 class OVNPortUpdateError(n_exc.BadRequest):
     pass
@@ -111,12 +118,14 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._plugin_property = None
         self._ovn_client_inst = None
         self._maintenance_thread = None
-        self._hash_ring_thread = None
-        self._hash_ring_probe_event = multiprocessing.Event()
         self.node_uuid = None
         self.hash_ring_group = ovn_const.HASH_RING_ML2_GROUP
         self.sg_enabled = ovn_acl.is_sg_enabled()
+        # NOTE(lucasagomes): _clean_hash_ring() must be called before
+        # self.subscribe() to avoid processes racing when adding or
+        # deleting nodes from the Hash Ring during service initialization
         ovn_conf.register_opts()
+        self._clean_hash_ring()
         self._post_fork_event = threading.Event()
         if cfg.CONF.SECURITYGROUP.firewall_driver:
             LOG.warning('Firewall driver configuration is ignored')
@@ -180,8 +189,7 @@ class OVNMechanismDriver(api.MechanismDriver):
     def get_supported_vif_types(self):
         vif_types = set()
         for ch in self.sb_ovn.chassis_list().execute(check_error=True):
-            other_config = ovn_utils.get_ovn_chassis_other_config(ch)
-            dp_type = other_config.get('datapath-type', '')
+            dp_type = ch.external_ids.get('datapath-type', '')
             if dp_type == ovn_const.CHASSIS_DATAPATH_NETDEV:
                 vif_types.add(portbindings.VIF_TYPE_VHOST_USER)
             else:
@@ -270,18 +278,56 @@ class OVNMechanismDriver(api.MechanismDriver):
                                resources.SECURITY_GROUP_RULE,
                                events.BEFORE_DELETE)
 
-    def _remove_node_from_hash_ring(self, *args, **kwargs):
-        # The node_uuid attribute will be empty for worker types
-        # that are not added to the Hash Ring and can be skipped
-        if self.node_uuid is None:
-            return
+    def _clean_hash_ring(self, *args, **kwargs):
         admin_context = n_context.get_admin_context()
-        ovn_hash_ring_db.remove_node_by_uuid(
-            admin_context, self.node_uuid)
+        ovn_hash_ring_db.remove_nodes_from_host(admin_context,
+                                                self.hash_ring_group)
 
     def pre_fork_initialize(self, resource, event, trigger, payload=None):
         """Pre-initialize the ML2/OVN driver."""
-        ovn_utils.create_neutron_pg_drop()
+        atexit.register(self._clean_hash_ring)
+        signal.signal(signal.SIGTERM, self._clean_hash_ring)
+        self._create_neutron_pg_drop()
+
+    def _create_neutron_pg_drop(self):
+        """Create neutron_pg_drop Port Group.
+
+        The method creates a short living connection to the Northbound
+        database. Because of multiple controllers can attempt to create the
+        Port Group at the same time the transaction can fail and raise
+        RuntimeError. In such case, we make sure the Port Group was created,
+        otherwise the error is something else and it's raised to the caller.
+        """
+        idl = ovsdb_monitor.OvnInitPGNbIdl.from_server(
+            ovn_conf.get_ovn_nb_connection(), self.nb_schema_helper, self)
+        # Only one server should try to create the port group
+        idl.set_lock('pg_drop_creation')
+        with ovsdb_monitor.short_living_ovsdb_api(
+                impl_idl_ovn.OvsdbNbOvnIdl, idl) as pre_ovn_nb_api:
+            try:
+                create_default_drop_port_group(pre_ovn_nb_api)
+            except KeyError:
+                # Due to a bug in python-ovs, we can send transactions before
+                # the initial OVSDB is populated in memory. This can break
+                # the AddCommand post_commit method which tries to return a
+                # row looked up by the newly commited row's uuid. Since we
+                # don't care about the return value from the PgAddCommand, we
+                # can just catch the KeyError and continue. This can be
+                # removed when the python-ovs bug is resolved.
+                pass
+            except RuntimeError as re:
+                # If we don't get the lock, and the port group didn't exist
+                # when we tried to create it, it might still have been
+                # created by another server and we just haven't gotten the
+                # update yet.
+                LOG.info("Waiting for Port Group %(pg)s to be created",
+                         {'pg': ovn_const.OVN_DROP_PORT_GROUP_NAME})
+                if not idl.neutron_pg_drop_event.wait():
+                    LOG.error("Port Group %(pg)s was not created in time",
+                              {'pg': ovn_const.OVN_DROP_PORT_GROUP_NAME})
+                    raise re
+                LOG.info("Porg Group %(pg)s was created by another server",
+                         {'pg': ovn_const.OVN_DROP_PORT_GROUP_NAME})
 
     @staticmethod
     def should_post_fork_initialize(worker_class):
@@ -289,37 +335,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                                 worker.MaintenanceWorker,
                                 service.RpcWorker)
 
-    @lockutils.synchronized('hash_ring_probe_lock', external=True)
-    def _setup_hash_ring(self):
-        """Setup the hash ring.
-
-        The first worker to acquire the lock is responsible for cleaning
-        the hash ring from previous runs as well as start the probing
-        thread for this host. Subsequently workers just need to register
-        themselves to the hash ring.
-        """
-        # Attempt to remove the node from the ring when the worker stops
-        atexit.register(self._remove_node_from_hash_ring)
-        signal.signal(signal.SIGTERM, self._remove_node_from_hash_ring)
-
-        admin_context = n_context.get_admin_context()
-        if not self._hash_ring_probe_event.is_set():
-            # Clear existing entries
-            ovn_hash_ring_db.remove_nodes_from_host(admin_context,
-                                                    self.hash_ring_group)
-            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
-                                                       self.hash_ring_group)
-            self._hash_ring_thread = maintenance.MaintenanceThread()
-            self._hash_ring_thread.add_periodics(
-                maintenance.HashRingHealthCheckPeriodics(
-                    self.hash_ring_group))
-            self._hash_ring_thread.start()
-            LOG.info("Hash Ring probing thread has started")
-            self._hash_ring_probe_event.set()
-        else:
-            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
-                                                       self.hash_ring_group)
-
     def post_fork_initialize(self, resource, event, trigger, payload=None):
         # Initialize API/Maintenance workers with OVN IDL connections
         worker_class = ovn_utils.get_method_class(trigger)
@@ -330,7 +345,9 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._ovn_client_inst = None
 
         if worker_class == neutron.wsgi.WorkerService:
-            self._setup_hash_ring()
+            admin_context = n_context.get_admin_context()
+            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
+                                                       self.hash_ring_group)
 
         n_agent.AgentCache(self)  # Initialize singleton agent cache
         self.nb_ovn, self.sb_ovn = impl_idl_ovn.get_ovn_idls(self, trigger)
@@ -375,8 +392,10 @@ class OVNMechanismDriver(api.MechanismDriver):
             self._maintenance_thread = maintenance.MaintenanceThread()
             self._maintenance_thread.add_periodics(
                 maintenance.DBInconsistenciesPeriodics(self._ovn_client))
+            self._maintenance_thread.add_periodics(
+                maintenance.HashRingHealthCheckPeriodics(
+                    self.hash_ring_group))
             self._maintenance_thread.start()
-            LOG.info("Maintenance task thread has started")
 
     def _create_security_group_precommit(self, resource, event, trigger,
                                          payload):
@@ -971,9 +990,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                                       'agent': agent})
             return
         chassis = agent.chassis
-        other_config = ovn_utils.get_ovn_chassis_other_config(chassis)
-        datapath_type = other_config.get('datapath-type', '')
-        iface_types = other_config.get('iface-types', '')
+        datapath_type = chassis.external_ids.get('datapath-type', '')
+        iface_types = chassis.external_ids.get('iface-types', '')
         iface_types = iface_types.split(',') if iface_types else []
         chassis_physnets = self.sb_ovn._get_chassis_physnets(chassis)
         for segment_to_bind in context.segments_to_bind:
@@ -1069,15 +1087,14 @@ class OVNMechanismDriver(api.MechanismDriver):
                                {ovn_const.OVN_FIP_EXT_MAC_KEY:
                                 nat['external_mac']})).execute()
 
-        if ovn_conf.is_ovn_distributed_floating_ip():
-            if up:
-                mac = nat['external_ids'].get(ovn_const.OVN_FIP_EXT_MAC_KEY)
-                if mac and nat['external_mac'] != mac:
-                    LOG.debug("Setting external_mac of port %s to %s",
-                              port_id, mac)
-                    self.nb_ovn.db_set(
-                        'NAT', nat['_uuid'], ('external_mac', mac)).execute(
-                            check_error=True)
+        if up and ovn_conf.is_ovn_distributed_floating_ip():
+            mac = nat['external_ids'][ovn_const.OVN_FIP_EXT_MAC_KEY]
+            if nat['external_mac'] != mac:
+                LOG.debug("Setting external_mac of port %s to %s",
+                          port_id, mac)
+                self.nb_ovn.db_set(
+                    'NAT', nat['_uuid'], ('external_mac', mac)).execute(
+                    check_error=True)
         else:
             if nat['external_mac']:
                 LOG.debug("Clearing up external_mac of port %s", port_id)
@@ -1113,7 +1130,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             # NOTE(lucasagomes): Router ports in OVN is never bound
             # to a host given their decentralized nature. By calling
             # provisioning_complete() - as above - don't do it for us
-            # becasue the router ports are unbind so, for OVN we are
+            # because the router ports are unbind so, for OVN we are
             # forcing the status here. Maybe it's something that we can
             # change in core Neutron in the future.
             db_port = ml2_db.get_port(admin_context, port_id)
@@ -1127,8 +1144,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                 const.PORT_STATUS_ACTIVE)
             elif self._should_notify_nova(db_port):
                 self._plugin.nova_notifier.notify_port_active_direct(db_port)
-
-            self._ovn_client.update_lsp_host_info(admin_context, db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug('Port not found during OVN status up report: %s',
                       port_id)
@@ -1157,26 +1172,25 @@ class OVNMechanismDriver(api.MechanismDriver):
                     None)
                 self._plugin.nova_notifier.send_port_status(
                     None, None, db_port)
-
-            self._ovn_client.update_lsp_host_info(
-                admin_context, db_port, up=False)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug("Port not found during OVN status down report: %s",
                       port_id)
 
     def delete_mac_binding_entries(self, external_ip):
         """Delete all MAC_Binding entries associated to this IP address"""
-        cmd = [
-            "OVN_Southbound", {
-                "op": "delete",
-                "table": "MAC_Binding",
-                "where": [
-                    ["ip", "==", external_ip]
-                ]
-            }
-        ]
+        cmd = ['ovsdb-client', 'transact', ovn_conf.get_ovn_sb_connection(),
+               '--timeout', str(ovn_conf.get_ovn_ovsdb_timeout())]
 
-        return ovn_utils.OvsdbClientTransactCommand.run(cmd)
+        if ovn_conf.get_ovn_sb_private_key():
+            cmd += ['-p', ovn_conf.get_ovn_sb_private_key(), '-c',
+                    ovn_conf.get_ovn_sb_certificate(), '-C',
+                    ovn_conf.get_ovn_sb_ca_cert()]
+
+        cmd += ['["OVN_Southbound", {"op": "delete", "table": "MAC_Binding", '
+                '"where": [["ip", "==", "%s"]]}]' % external_ip]
+
+        return processutils.execute(*cmd,
+                                    log_errors=processutils.LOG_FINAL_ERROR)
 
     def update_segment_host_mapping(self, host, phy_nets):
         """Update SegmentHostMapping in DB"""
@@ -1286,6 +1300,58 @@ class OVNMechanismDriver(api.MechanismDriver):
                               'tenant_id': context.project_id}
         return azs
 
+    def responsible_for_ports_allocation(self, context):
+        """Report if a chassis is responsible for a resource provider.
+
+        New "allocation" resource request defined in
+        https://review.opendev.org/c/openstack/neutron-specs/+/785236
+
+        This method replicates the logic implemented in
+        ``AgentMechanismDriverBase.responsible_for_ports_allocation``.
+
+        :param context: PortContext instance describing the port
+        :returns: True for responsible, False for not responsible
+        """
+        uuid_ns = self.resource_provider_uuid5_namespace
+        if uuid_ns is None:
+            return False
+        try:
+            allocation = context.current['binding:profile'][ALLOCATION]
+        except KeyError:
+            return False
+
+        reported = collections.defaultdict(list)
+        _placement = self._ovn_client.placement_extension
+        for ch_name, state in _placement.get_chassis_config().items():
+            for device in state._rp_bandwidths:
+                # chassis = {RP_HYPERVISORS:
+                #     {device1: {name: hostname1, uuid: uuid1}
+                #     {device2: {name: hostname1, uuid: uuid2} ...}
+                hostname = state._hypervisor_rps.get(device, {}).get('name')
+                if not hostname:
+                    continue
+                device_rp_uuid = place_utils.device_resource_provider_uuid(
+                    namespace=uuid_ns, host=hostname, device=device)
+                for group, rp in allocation.items():
+                    if device_rp_uuid == uuid.UUID(rp):
+                        reported[group].append((ch_name, state))
+
+        for group, states in reported.items():
+            if len(states) == 1:
+                ch_name = states[0][0]
+                ch_rp = placement_ext.dict_chassis_config(states[0][1])
+                LOG.debug('Chassis %s is reponsible of the resource provider '
+                          '%s', ch_name, ch_rp)
+                return True
+            elif len(states) > 1:
+                rps = {state[0]: placement_ext.dict_chassis_config(state[1])
+                       for state in states}
+                LOG.error('Several chassis reported the requested resource '
+                          'provider: %s', rps)
+                return False
+
+        return False
+
 
 def get_agents(self, context, filters=None, fields=None, _driver=None):
     _driver.ping_all_chassis()
@@ -1337,14 +1403,6 @@ def delete_agent(self, context, id, _driver=None):
     chassis_name = agent['configurations']['chassis_name']
     _driver.sb_ovn.chassis_del(chassis_name, if_exists=True).execute(
         check_error=True)
-    if _driver.sb_ovn.is_table_present('Chassis_Private'):
-        # TODO(ralonsoh): implement the corresponding chassis_private
-        # commands in ovsdbapp.
-        try:
-            _driver.sb_ovn.db_destroy('Chassis_Private', chassis_name).execute(
-                check_error=True)
-        except idlutils.RowNotFound:
-            pass
     # Send a specific event that all API workers can get to delete the agent
     # from their caches. Ideally we could send a single transaction that both
     # created and deleted the key, but alas python-ovs is too "smart"
@@ -1356,6 +1414,28 @@ def delete_agent(self, context, id, _driver=None):
         if_exists=True).execute(check_error=True)
 
 
+def create_default_drop_port_group(nb_idl):
+    pg_name = ovn_const.OVN_DROP_PORT_GROUP_NAME
+    if nb_idl.get_port_group(pg_name):
+        LOG.debug("Port Group %s already exists", pg_name)
+        return
+    with nb_idl.transaction(check_error=True) as txn:
+        # If drop Port Group doesn't exist yet, create it.
+        txn.add(nb_idl.pg_add(pg_name, acls=[], may_exist=True))
+        # Add ACLs to this Port Group so that all traffic is dropped.
+        acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
+        for acl in acls:
+            txn.add(nb_idl.pg_acl_add(may_exist=True, **acl))
+
+        ports_with_pg = set()
+        for pg in nb_idl.get_sg_port_groups().values():
+            ports_with_pg.update(pg['ports'])
+
+        if ports_with_pg:
+            # Add the ports to the default Port Group
+            txn.add(nb_idl.pg_add_ports(pg_name, list(ports_with_pg)))
+
+
 def get_availability_zones(cls, context, _driver, filters=None, fields=None,
                            sorts=None, limit=None, marker=None,
                            page_reverse=False):
