@@ -11,7 +11,6 @@
 #    under the License.
 
 from collections import namedtuple
-import random
 
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import resources
@@ -28,7 +27,6 @@ from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.conf.services import logging as log_cfg
-from neutron.objects import securitygroup as sg_obj
 from neutron.services.logapi.common import db_api
 from neutron.services.logapi.common import sg_callback
 from neutron.services.logapi.drivers import base
@@ -40,7 +38,6 @@ DRIVER = None
 
 log_cfg.register_log_driver_opts()
 
-MAX_INT_LABEL = 2**32
 SUPPORTED_LOGGING_TYPES = [log_const.SECURITY_GROUP]
 
 
@@ -90,14 +87,54 @@ class OVNDriver(base.DriverBase):
         log_objs = self._log_plugin.get_logs(context)
         return [self._log_dict_to_obj(lo) for lo in log_objs]
 
-    @property
-    def _ovn_client(self):
-        return self.plugin_driver._ovn_client
-
     @property
     def ovn_nb(self):
         return self.plugin_driver.nb_ovn
 
+    def _create_ovn_fair_meter(self, ovn_txn):
+        """Create row in Meter table with fair attribute set to True.
+
+        Create a row in OVN's NB Meter table based on well-known name. This
+        method uses the network_log configuration to specify the attributes
+        of the meter. Current implementation needs only one 'fair' meter row
+        which is then referred by multiple ACL rows.
+
+        :param ovn_txn: ovn northbound idl transaction.
+
+        """
+        meter = self.ovn_nb.db_find_rows(
+            "Meter", ("name", "=", self.meter_name)).execute(check_error=True)
+        if meter:
+            meter = meter[0]
+            try:
+                meter_band = self.ovn_nb.lookup("Meter_Band",
+                                                meter.bands[0].uuid)
+                if all((meter.unit == "pktps",
+                        meter.fair[0],
+                        meter_band.rate == cfg.CONF.network_log.rate_limit,
+                        meter_band.burst_size ==
+                        cfg.CONF.network_log.burst_limit)):
+                    # Meter (and its meter-band) unchanged: noop.
+                    return
+            except idlutils.RowNotFound:
+                pass
+            # Re-create meter (and its meter-band) with the new attributes.
+            # This is supposed to happen only if configuration changed, so
+            # doing updates is an overkill: better to leverage the ovsdbapp
+            # library to avoid the complexity.
+            ovn_txn.add(self.ovn_nb.meter_del(meter.uuid))
+        # Create meter
+        LOG.info("Creating network log fair meter %s", self.meter_name)
+        ovn_txn.add(self.ovn_nb.meter_add(
+            name=self.meter_name,
+            unit="pktps",
+            rate=cfg.CONF.network_log.rate_limit,
+            fair=True,
+            burst_size=cfg.CONF.network_log.burst_limit,
+            may_exist=False,
+            external_ids={ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                          log_const.LOGGING_PLUGIN}))
+
     @staticmethod
     def _acl_actions_enabled(log_obj):
         if not log_obj.enabled:
@@ -132,62 +169,35 @@ class OVNDriver(base.DriverBase):
                 if log_name:
                     if acl.name and acl.name[0] != log_name:
                         continue
-                columns = {
-                    'log': False,
-                    'meter': [],
-                    'name': [],
-                    'severity': []
-                }
-                # TODO(egarciar): There wont be a need to check if label exists
-                # once minimum version for OVN is >= 22.03
-                if hasattr(acl, 'label'):
-                    columns['label'] = 0
-                    ovn_txn.add(self.ovn_nb.db_remove(
-                        "ACL", acl_uuid, 'options', 'log-related',
-                        if_exists=True))
                 ovn_txn.add(self.ovn_nb.db_set(
-                    "ACL", acl_uuid, *columns.items()))
+                    "ACL", acl_uuid,
+                    ("log", False),
+                    ("meter", []),
+                    ("name", []),
+                    ("severity", [])
+                ))
                 acl_changes += 1
         msg = "Cleared %d, Not found %d (out of %d visited) ACLs"
         if log_name:
             msg += " for network log {}".format(log_name)
         LOG.info(msg, acl_changes, acl_absents, acl_visits)
 
-    def _set_acls_log(self, pgs, context, ovn_txn, actions_enabled, log_name):
+    def _set_acls_log(self, pgs, ovn_txn, actions_enabled, log_name):
         acl_changes, acl_visits = 0, 0
         for pg in pgs:
-            meter_name = self.meter_name
-            if pg["name"] != ovn_const.OVN_DROP_PORT_GROUP_NAME:
-                sg = sg_obj.SecurityGroup.get_sg_by_id(
-                    context,
-                    pg["external_ids"][ovn_const.OVN_SG_EXT_ID_KEY])
-                if not sg:
-                    LOG.warning("Port Group %s is missing a corresponding "
-                                "security group, skipping its network log "
-                                "setting...", pg["name"])
-                    continue
-                if not sg.stateful:
-                    meter_name = meter_name + ("_stateless")
             for acl_uuid in pg["acls"]:
                 acl_visits += 1
                 acl = self.ovn_nb.lookup("ACL", acl_uuid)
                 # skip acls used by a different network log
                 if acl.name and acl.name[0] != log_name:
                     continue
-                columns = {
-                    'log': acl.action in actions_enabled,
-                    'meter': meter_name,
-                    'name': log_name,
-                    'severity': "info"
-                }
-                # TODO(egarciar): There wont be a need to check if label exists
-                # once minimum version for OVN is >= 22.03
-                if hasattr(acl, "label"):
-                    # Label needs to be an unsigned 32 bit number and not 0.
-                    columns["label"] = random.randrange(1, MAX_INT_LABEL)
-                    columns["options"] = {'log-related': "true"}
                 ovn_txn.add(self.ovn_nb.db_set(
-                    "ACL", acl_uuid, *columns.items()))
+                    "ACL", acl_uuid,
+                    ("log", acl.action in actions_enabled),
+                    ("meter", self.meter_name),
+                    ("name", log_name),
+                    ("severity", "info")
+                ))
                 acl_changes += 1
         LOG.info("Set %d (out of %d visited) ACLs for network log %s",
                  acl_changes, acl_visits, log_name)
@@ -196,13 +206,12 @@ class OVNDriver(base.DriverBase):
         for log_obj in log_objs:
             pgs = self._pgs_from_log_obj(context, log_obj)
             actions_enabled = self._acl_actions_enabled(log_obj)
-            self._set_acls_log(pgs, context, ovn_txn, actions_enabled,
+            self._set_acls_log(pgs, ovn_txn, actions_enabled,
                                utils.ovn_name(log_obj.id))
 
     def _pgs_all(self):
         return self.ovn_nb.db_list(
-            "Port_Group",
-            columns=["name", "external_ids", "acls"]).execute(check_error=True)
+            "Port_Group", columns=["name", "acls"]).execute(check_error=True)
 
     def _pgs_from_log_obj(self, context, log_obj):
         """Map Neutron log_obj into affected port groups in OVN.
@@ -221,13 +230,11 @@ class OVNDriver(base.DriverBase):
                 # No sg, no port, DROP: return DROP pg
                 if log_obj.event == log_const.DROP_EVENT:
                     return [{"name": pg_drop.name,
-                             "external_ids": pg_drop.external_ids,
-                             "acls": [r.uuid for r in pg_drop.acls]}]
+                        "acls": [r.uuid for r in pg_drop.acls]}]
                 # No sg, no port, ACCEPT: return all except DROP pg
                 pgs = self._pgs_all()
                 pgs.remove({"name": pg_drop.name,
-                            "external_ids": pg_drop.external_ids,
-                            "acls": [r.uuid for r in pg_drop.acls]})
+                    "acls": [r.uuid for r in pg_drop.acls]})
                 return pgs
             except idlutils.RowNotFound:
                 pass
@@ -239,7 +246,6 @@ class OVNDriver(base.DriverBase):
                 pg = self.ovn_nb.lookup("Port_Group",
                                         ovn_const.OVN_DROP_PORT_GROUP_NAME)
                 pgs.append({"name": pg.name,
-                            "external_ids": pg.external_ids,
                             "acls": [r.uuid for r in pg.acls]})
             except idlutils.RowNotFound:
                 pass
@@ -252,7 +258,6 @@ class OVNDriver(base.DriverBase):
                                         utils.ovn_port_group_name(
                                             log_obj.resource_id))
                 pgs.append({"name": pg.name,
-                            "external_ids": pg.external_ids,
                             "acls": [r.uuid for r in pg.acls]})
             except idlutils.RowNotFound:
                 pass
@@ -266,7 +271,6 @@ class OVNDriver(base.DriverBase):
                     pg = self.ovn_nb.lookup("Port_Group",
                                             utils.ovn_port_group_name(sg_id))
                     pgs.append({"name": pg.name,
-                                "external_ids": pg.external_ids,
                                 "acls": [r.uuid for r in pg.acls]})
                 except idlutils.RowNotFound:
                     pass
@@ -283,9 +287,8 @@ class OVNDriver(base.DriverBase):
         pgs = self._pgs_from_log_obj(context, log_obj)
         actions_enabled = self._acl_actions_enabled(log_obj)
         with self.ovn_nb.transaction(check_error=True) as ovn_txn:
-            self._ovn_client.create_ovn_fair_meter(self.meter_name,
-                                                   txn=ovn_txn)
-            self._set_acls_log(pgs, context, ovn_txn, actions_enabled,
+            self._create_ovn_fair_meter(ovn_txn)
+            self._set_acls_log(pgs, ovn_txn, actions_enabled,
                                utils.ovn_name(log_obj.id))
 
     def create_log_precommit(self, context, log_obj):
@@ -299,46 +302,6 @@ class OVNDriver(base.DriverBase):
         if not self.network_logging_supported(self.ovn_nb):
             raise LoggingNotSupported()
 
-    def _unset_disabled_acls(self, context, log_obj, ovn_txn):
-        """Check if we need to disable any ACLs after an update.
-
-        Will return True if there were more logs, and False if there was
-        nothing to check.
-
-        :param context: current running context information
-        :param log_obj: a log_object which was updated
-        :returns: True if there were other logs enabled, otherwise False.
-        """
-        if log_obj.enabled:
-            return False
-
-        pgs = self._pgs_from_log_obj(context, log_obj)
-        other_logs = [log for log in self._get_logs(context)
-                      if log.id != log_obj.id and log.enabled]
-        if not other_logs:
-            return False
-
-        if log_obj.event == log_const.ALL_EVENT:
-            acls_to_check = pgs[0]["acls"].copy()
-            if not acls_to_check:
-                return True
-            for log in other_logs:
-                for acl in self._pgs_from_log_obj(context, log)[0]["acls"]:
-                    if acl in acls_to_check:
-                        acls_to_check.remove(acl)
-                    if not acls_to_check:
-                        return True
-            acls_to_remove = [{"name": pgs[0]["name"], "acls": acls_to_check}]
-            self._remove_acls_log(acls_to_remove, ovn_txn)
-        else:
-            all_events = set([log.event for log in other_logs
-                if (not log.resource_id or
-                    log.resource_id == log_obj.resource_id)])
-            if (log_const.ALL_EVENT not in all_events and
-                    log_obj.event not in all_events):
-                self._remove_acls_log(pgs, ovn_txn)
-        return True
-
     def update_log(self, context, log_obj):
         """Update a log_obj invocation.
 
@@ -348,13 +311,11 @@ class OVNDriver(base.DriverBase):
         """
         LOG.debug("Update_log %s", log_obj)
 
+        pgs = self._pgs_from_log_obj(context, log_obj)
+        actions_enabled = self._acl_actions_enabled(log_obj)
         with self.ovn_nb.transaction(check_error=True) as ovn_txn:
-
-            if not self._unset_disabled_acls(context, log_obj, ovn_txn):
-                pgs = self._pgs_from_log_obj(context, log_obj)
-                actions_enabled = self._acl_actions_enabled(log_obj)
-                self._set_acls_log(pgs, context, ovn_txn, actions_enabled,
-                                   utils.ovn_name(log_obj.id))
+            self._set_acls_log(pgs, ovn_txn, actions_enabled,
+                               utils.ovn_name(log_obj.id))
 
     def delete_log(self, context, log_obj):
         """Delete a log_obj invocation.
@@ -375,8 +336,6 @@ class OVNDriver(base.DriverBase):
                 self._remove_acls_log(pgs, ovn_txn)
                 ovn_txn.add(self.ovn_nb.meter_del(self.meter_name,
                                                   if_exists=True))
-                ovn_txn.add(self.ovn_nb.meter_del(
-                    self.meter_name + "_stateless", if_exists=True))
             LOG.info("All ACL logs cleared after deletion of log_obj %s",
                      log_obj.id)
             return
